;; This code was written by Stefan Kamphausen <http://www.skamphausen.de>
;; If accepted by the SLIME maintainers copyright will be passed to
;; them such that it can be added to the SLIME project in whatever way
;; those maintainers need.  A note in the ChangeLog and the eternal
;; fame gathered from that will be sufficient for the author ;-)
;; Licensing is not an issue.


(defvar slime-repl-history-file "~/.slime-history.el")
(defvar slime-repl-history-size 100)

(defun slime-repl-merge-histories (old-hist new-hist)
  "Merge entries from OLD-HIST and NEW-HIST such that the new items in
  NEW-HIST are appended to the OLD-HIST."
  (append
   ;; first the new unique elements...
   (remove-if #'(lambda (entry)
                  (member entry old-hist))
              new-hist)
   ;; then the old unique elements...
   (remove-if #'(lambda (entry)
                  (member entry new-hist))
              old-hist)
   ;; and finally elements existing in both lists
   (remove-if #'(lambda (entry)
                  (not (member entry old-hist)))
              new-hist)))

(defun slime-repl-read-history-internal (filename)
  "Return the list stored in FILENAME.
The file contents are read using READ and no further error checking is
done."
  (when (and file (file-readable-p file))
    (with-temp-buffer
      (insert-file-contents file)
      (read (current-buffer)))))

(defun slime-repl-read-history (&optional filename)
  "Set the current SLIME REPL history.
It can be read either from FILENAME or `slime-repl-history-file' or
from a user defined filename."
  (interactive)
  (let ((file (or filename
                  slime-repl-history-file
                  (read-file-name "Read SLIME REPL history from file: "))))
    (setq slime-repl-input-history
            (slime-repl-read-history-internal file))))
  
(defun slime-repl-save-merged-history (&optional filename)
  "Read the history file, merge the current REPL history and save it.
This tries to be smart in merging the history from the file and the
current history in that it tries to detect the unique entries using
`slime-repl-merge-histories'."
  (interactive)
  (let ((file (or filename
                  slime-repl-history-file
                  (read-file-name "Save SLIME REPL history to file: "))))
    (cond
      ((or (null file)
           (null slime-repl-input-history))
        nil)
      ((not (file-writable-p file))
        (error (format "Can't write SLIME REPL history file %s" file)))
      (t
        (let ((hist (slime-repl-read-history-internal file)))
          (if (not (null hist))
            (setq hist (slime-repl-merge-histories
                        hist slime-repl-input-history))
            (setq hist slime-repl-input-history))
          (slime-repl-save-history hist file))))))

(defun slime-repl-save-history (&optional history filename)
  "Simply save the current SLIME REPL history to a file.
When SLIME is setup to always load the old history and one uses only
one instance of slime all the time, there is no need to merge the
files and this function is sufficient.

When the list is longer than `slime-repl-history-size' it will be
truncated.  That part is untested, though!
"
  (interactive)
  (let ((file (or filename
                  slime-repl-history-file
                  (read-file-name "Save SLIME REPL history to file: "))))
    (cond
      ((or (null file)
           (null slime-repl-input-history))
        nil)
      ((not (file-writable-p file))
        (error (format "Can't write SLIME REPL history file %s" file)))
      (t
        (let* ((hist (or history slime-repl-input-history))
               (len (length hist)))
          (when (> len slime-repl-history-size)
            (setq hist (subseq hist (- len slime-repl-history-size))))
          (message "saving %s to %s\n" hist file)
          (with-temp-buffer
            (insert ";; History for SLIME REPL.  Automatically written\n")
            (insert ";; Edit only if you know what you're doing\n")
            (pp hist (current-buffer))
            (write-region (point-min) (point-max) file)))))))
          
